@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  * Authors: Chris Toshok <toshok@ximian.com>
  */
@@ -56,7 +56,8 @@
 
 #define d(x) x
 
-static EBookBackendSyncClass *e_book_backend_vcf_parent_class;
+G_DEFINE_TYPE (EBookBackendVCF, e_book_backend_vcf, E_TYPE_BOOK_BACKEND_SYNC);
+
 typedef struct _EBookBackendVCFBookView EBookBackendVCFBookView;
 typedef struct _EBookBackendVCFSearchContext EBookBackendVCFSearchContext;
 
@@ -67,15 +68,10 @@ struct _EBookBackendVCFPrivate {
 	GList      *contact_list;
 	gboolean    dirty;
 	int         flush_timeout_tag;
-	/* for future use */
-	void *reserved1;
-	void *reserved2;
-	void *reserved3;
-	void *reserved4;
 };
 
 static char *
-e_book_backend_vcf_create_unique_id ()
+e_book_backend_vcf_create_unique_id (void)
 {
 	/* use a 32 counter and the 32 bit timestamp to make an id.
 	   it's doubtful 2^32 id's will be created in a second, so we
@@ -303,23 +299,19 @@ e_book_backend_vcf_remove_contacts (EBookBackendSync *backend,
 	EBookBackendVCF *bvcf = E_BOOK_BACKEND_VCF (backend);
 	char *id = id_list->data;
 	GList *elem;
-	gboolean success;
-	char *real_id;
 
 	g_mutex_lock (bvcf->priv->mutex);
-	success = g_hash_table_lookup_extended (bvcf->priv->contacts, id, (gpointer)&real_id, (gpointer)&elem);
-	if (!success) {
+	elem = g_hash_table_lookup (bvcf->priv->contacts, id);
+	if (!elem) {
 		g_mutex_unlock (bvcf->priv->mutex);
 		return GNOME_Evolution_Addressbook_ContactNotFound;
 	}
 
-	success = g_hash_table_remove (bvcf->priv->contacts, real_id);
-	if (!success) {
+	if (!g_hash_table_remove (bvcf->priv->contacts, id)) {
 		g_mutex_unlock (bvcf->priv->mutex);
 		return GNOME_Evolution_Addressbook_ContactNotFound;
 	}
 
-	g_free (real_id);
 	g_free (elem->data);
 	bvcf->priv->contact_list = g_list_remove_link (bvcf->priv->contact_list, elem);
 
@@ -342,19 +334,16 @@ e_book_backend_vcf_modify_contact (EBookBackendSync *backend,
 				   EContact **contact)
 {
 	EBookBackendVCF *bvcf = E_BOOK_BACKEND_VCF (backend);
-	char *old_id;
 	GList *elem;
 	const char *id;
-	gboolean success;
 
 	/* create a new ecard from the request data */
 	*contact = e_contact_new_from_vcard (vcard);
 	id = e_contact_get_const (*contact, E_CONTACT_UID);
 
 	g_mutex_lock (bvcf->priv->mutex);
-	success = g_hash_table_lookup_extended (bvcf->priv->contacts, id, (gpointer)&old_id, (gpointer)&elem);
-
-	if (!success) {
+	elem = g_hash_table_lookup (bvcf->priv->contacts, id);
+	if (!elem) {
 		g_mutex_unlock (bvcf->priv->mutex);
 		return GNOME_Evolution_Addressbook_ContactNotFound;
 	}
@@ -615,7 +604,10 @@ e_book_backend_vcf_load_source (EBookBackend             *backend,
 
 	fd = g_open (bvcf->priv->filename, O_RDWR | O_BINARY, 0);
 
-	bvcf->priv->contacts = g_hash_table_new (g_str_hash, g_str_equal);
+	bvcf->priv->contacts = g_hash_table_new_full (
+		g_str_hash, g_str_equal,
+		(GDestroyNotify) g_free,
+		(GDestroyNotify) NULL);
 
 	if (fd != -1) {
 		writable = TRUE;
@@ -741,7 +733,6 @@ e_book_backend_vcf_dispose (GObject *object)
 		if (bvcf->priv->dirty)
 			save_file (bvcf);
 
-		g_hash_table_foreach (bvcf->priv->contacts, (GHFunc)g_free, NULL);
 		g_hash_table_destroy (bvcf->priv->contacts);
 		g_list_foreach (bvcf->priv->contact_list, (GFunc)g_free, NULL);
 		g_list_free (bvcf->priv->contact_list);
@@ -766,8 +757,6 @@ e_book_backend_vcf_class_init (EBookBackendVCFClass *klass)
 	EBookBackendSyncClass *sync_class;
 	EBookBackendClass *backend_class;
 
-	e_book_backend_vcf_parent_class = g_type_class_peek_parent (klass);
-
 	sync_class = E_BOOK_BACKEND_SYNC_CLASS (klass);
 	backend_class = E_BOOK_BACKEND_CLASS (klass);
 
@@ -800,30 +789,3 @@ e_book_backend_vcf_init (EBookBackendVCF *backend)
 
 	backend->priv = priv;
 }
-
-/**
- * e_book_backend_vcf_get_type:
- */
-GType
-e_book_backend_vcf_get_type (void)
-{
-	static GType type = 0;
-
-	if (! type) {
-		GTypeInfo info = {
-			sizeof (EBookBackendVCFClass),
-			NULL, /* base_class_init */
-			NULL, /* base_class_finalize */
-			(GClassInitFunc)  e_book_backend_vcf_class_init,
-			NULL, /* class_finalize */
-			NULL, /* class_data */
-			sizeof (EBookBackendVCF),
-			0,    /* n_preallocs */
-			(GInstanceInitFunc) e_book_backend_vcf_init
-		};
-
-		type = g_type_register_static (E_TYPE_BOOK_BACKEND_SYNC, "EBookBackendVCF", &info, 0);
-	}
-
-	return type;
-}
