@@ -7,23 +7,23 @@
  * Authors: Sivaiah Nallagatla <snallagatla@ximian.com>
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifndef E_BOOK_BACKEND_CACHE_H
 #define E_BOOK_BACKEND_CACHE_H
 
-#include <libedataserver/e-file-cache.h>
+#include "libedataserver/e-file-cache.h"
 #include <libebook/e-contact.h>
 
 G_BEGIN_DECLS
@@ -57,6 +57,8 @@ GList*   e_book_backend_cache_get_contacts (EBookBackendCache *cache, const char
 gboolean e_book_backend_cache_exists (const char *uri);
 void     e_book_backend_cache_set_populated (EBookBackendCache *cache);
 gboolean e_book_backend_cache_is_populated (EBookBackendCache *cache);
+void e_book_backend_cache_set_time (EBookBackendCache *cache, const char *t);
+char *e_book_backend_cache_get_time (EBookBackendCache *cache);
 GPtrArray* e_book_backend_cache_search (EBookBackendCache *cache, const char *query);
 
 
