@@ -6,7 +6,7 @@
  *           Jeffrey Stedfast <fejj@ximian.com>
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,40 +14,43 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+/* POSIX requires <sys/types.h> be included before <regex.h> */
+#include <sys/types.h>
+
+#include <ctype.h>
+#include <errno.h>
+#include <fcntl.h>
+#include <pthread.h>
+#include <regex.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/types.h>
+#include <time.h>
+#include <unistd.h>
 #include <sys/param.h>  /* for MAXHOSTNAMELEN */
 #include <sys/stat.h>
-#include <pthread.h>
-#include <unistd.h>
-#include <regex.h>
-#include <fcntl.h>
-#include <errno.h>
-#include <ctype.h>
-#include <time.h>
 
 #ifndef MAXHOSTNAMELEN
 #define MAXHOSTNAMELEN 1024
 #endif
 
 #include <glib.h>
+
 #include <libedataserver/e-iconv.h>
 #include <libedataserver/e-time-utils.h>
 
-#include "camel-mime-utils.h"
 #include "camel-charset-map.h"
+#include "camel-mime-utils.h"
 #include "camel-net-utils.h"
 #include "camel-utf8.h"
 
@@ -55,6 +58,16 @@
 #include "broken-date-parser.h"
 #endif
 
+#ifdef G_OS_WIN32
+/* Undef the similar macro from pthread.h, it doesn't check if
+ * gmtime() returns NULL.
+ */
+#undef gmtime_r
+
+/* The gmtime() in Microsoft's C library is MT-safe */
+#define gmtime_r(tp,tmp) (gmtime(tp)?(*(tmp)=*gmtime(tp),(tmp)):0)
+#endif
+
 #if 0
 int strdup_count = 0;
 int malloc_count = 0;
@@ -74,169 +87,60 @@ int free_count = 0;
 #define CAMEL_UUENCODE_CHAR(c)  ((c) ? (c) + ' ' : '`')
 #define	CAMEL_UUDECODE_CHAR(c)	(((c) - ' ') & 077)
 
-static char *base64_alphabet =
-"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
-
-static unsigned char tohex[16] = {
+static const unsigned char tohex[16] = {
 	'0', '1', '2', '3', '4', '5', '6', '7',
 	'8', '9', 'A', 'B', 'C', 'D', 'E', 'F'
 };
 
-static unsigned char camel_mime_base64_rank[256] = {
-	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-	0xff, 0xff, 0xff, 0x3e, 0xff, 0xff, 0xff, 0x3f,
-	0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3a, 0x3b,
-	0x3c, 0x3d, 0xff, 0xff, 0xff, 0x00, 0xff, 0xff,
-	0xff, 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06,
-	0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e,
-	0x0f, 0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16,
-	0x17, 0x18, 0x19, 0xff, 0xff, 0xff, 0xff, 0xff,
-	0xff, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f, 0x20,
-	0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28,
-	0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e, 0x2f, 0x30,
-	0x31, 0x32, 0x33, 0xff, 0xff, 0xff, 0xff, 0xff,
-	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-};
-
-/* call this when finished encoding everything, to
-   flush off the last little bit */
+/**
+ * camel_base64_encode_close:
+ * @in: input stream
+ * @inlen: length of the input
+ * @break_lines: whether or not to break long lines
+ * @out: output string
+ * @state: holds the number of bits that are stored in @save
+ * @save: leftover bits that have not yet been encoded
+ *
+ * Base64 encodes the input stream to the output stream. Call this
+ * when finished encoding data with #camel_base64_encode_step
+ * to flush off the last little bit.
+ *
+ * Returns the number of bytes encoded
+ **/
 size_t
 camel_base64_encode_close(unsigned char *in, size_t inlen, gboolean break_lines, unsigned char *out, int *state, int *save)
 {
-	int c1, c2;
-	unsigned char *outptr = out;
+	gsize bytes = 0;
 
-	if (inlen>0)
-		outptr += camel_base64_encode_step(in, inlen, break_lines, outptr, state, save);
+	if (inlen > 0)
+		bytes += g_base64_encode_step (in, inlen, break_lines, out, state, save);
 
-	c1 = ((unsigned char *)save)[1];
-	c2 = ((unsigned char *)save)[2];
-	
-	d(printf("mode = %d\nc1 = %c\nc2 = %c\n",
-		 (int)((char *)save)[0],
-		 (int)((char *)save)[1],
-		 (int)((char *)save)[2]));
+	bytes += g_base64_encode_close (break_lines, out, state, save);
 
-	switch (((char *)save)[0]) {
-	case 2:
-		outptr[2] = base64_alphabet[ ( (c2 &0x0f) << 2 ) ];
-		g_assert(outptr[2] != 0);
-		goto skip;
-	case 1:
-		outptr[2] = '=';
-	skip:
-		outptr[0] = base64_alphabet[ c1 >> 2 ];
-		outptr[1] = base64_alphabet[ c2 >> 4 | ( (c1&0x3) << 4 )];
-		outptr[3] = '=';
-		outptr += 4;
-		break;
-	}
-	if (break_lines)
-		*outptr++ = '\n';
-
-	*save = 0;
-	*state = 0;
-
-	return outptr-out;
+	return bytes;
 }
 
-/*
-  performs an 'encode step', only encodes blocks of 3 characters to the
-  output at a time, saves left-over state in state and save (initialise to
-  0 on first invocation).
-*/
+
+/**
+ * camel_base64_encode_step:
+ * @in: input stream
+ * @inlen: length of the input
+ * @break_lines: break long lines
+ * @out: output string
+ * @state: holds the number of bits that are stored in @save
+ * @save: leftover bits that have not yet been encoded
+ *
+ * Base64 encodes a chunk of data. Performs an 'encode step', only
+ * encodes blocks of 3 characters to the output at a time, saves
+ * left-over state in state and save (initialise to 0 on first
+ * invocation).
+ *
+ * Returns the number of bytes encoded
+ **/
 size_t
 camel_base64_encode_step(unsigned char *in, size_t len, gboolean break_lines, unsigned char *out, int *state, int *save)
 {
-	register unsigned char *inptr, *outptr;
-
-	if (len<=0)
-		return 0;
-
-	inptr = in;
-	outptr = out;
-
-	d(printf("we have %d chars, and %d saved chars\n", len, ((char *)save)[0]));
-
-	if (len + ((char *)save)[0] > 2) {
-		unsigned char *inend = in+len-2;
-		register int c1, c2, c3;
-		register int already;
-
-		already = *state;
-
-		switch (((char *)save)[0]) {
-		case 1:	c1 = ((unsigned char *)save)[1]; goto skip1;
-		case 2:	c1 = ((unsigned char *)save)[1];
-			c2 = ((unsigned char *)save)[2]; goto skip2;
-		}
-		
-		/* yes, we jump into the loop, no i'm not going to change it, it's beautiful! */
-		while (inptr < inend) {
-			c1 = *inptr++;
-		skip1:
-			c2 = *inptr++;
-		skip2:
-			c3 = *inptr++;
-			*outptr++ = base64_alphabet[ c1 >> 2 ];
-			*outptr++ = base64_alphabet[ c2 >> 4 | ( (c1&0x3) << 4 ) ];
-			*outptr++ = base64_alphabet[ ( (c2 &0x0f) << 2 ) | (c3 >> 6) ];
-			*outptr++ = base64_alphabet[ c3 & 0x3f ];
-			/* this is a bit ugly ... */
-			if (break_lines && (++already)>=19) {
-				*outptr++='\n';
-				already = 0;
-			}
-		}
-
-		((char *)save)[0] = 0;
-		len = 2-(inptr-inend);
-		*state = already;
-	}
-
-	d(printf("state = %d, len = %d\n",
-		 (int)((char *)save)[0],
-		 len));
-
-	if (len>0) {
-		register char *saveout;
-
-		/* points to the slot for the next char to save */
-		saveout = & (((char *)save)[1]) + ((char *)save)[0];
-
-		/* len can only be 0 1 or 2 */
-		switch(len) {
-		case 2:	*saveout++ = *inptr++;
-		case 1:	*saveout++ = *inptr++;
-		}
-		((char *)save)[0]+=len;
-	}
-
-	d(printf("mode = %d\nc1 = %c\nc2 = %c\n",
-		 (int)((char *)save)[0],
-		 (int)((char *)save)[1],
-		 (int)((char *)save)[2]));
-
-	return outptr-out;
+	return g_base64_encode_step (in, len, break_lines, out, state, save);
 }
 
 
@@ -249,81 +153,58 @@ camel_base64_encode_step(unsigned char *in, size_t len, gboolean break_lines, un
  * @save: leftover bits that have not yet been decoded
  *
  * Decodes a chunk of base64 encoded data
+ *
+ * Returns the number of bytes decoded (which have been dumped in @out)
  **/
 size_t
 camel_base64_decode_step(unsigned char *in, size_t len, unsigned char *out, int *state, unsigned int *save)
 {
-	register unsigned char *inptr, *outptr;
-	unsigned char *inend, c;
-	register unsigned int v;
-	int i;
-
-	inend = in+len;
-	outptr = out;
-
-	/* convert 4 base64 bytes to 3 normal bytes */
-	v=*save;
-	i=*state;
-	inptr = in;
-	while (inptr<inend) {
-		c = camel_mime_base64_rank[*inptr++];
-		if (c != 0xff) {
-			v = (v<<6) | c;
-			i++;
-			if (i==4) {
-				*outptr++ = v>>16;
-				*outptr++ = v>>8;
-				*outptr++ = v;
-				i=0;
-			}
-		}
-	}
-
-	*save = v;
-	*state = i;
-
-	/* quick scan back for '=' on the end somewhere */
-	/* fortunately we can drop 1 output char for each trailing = (upto 2) */
-	i=2;
-	while (inptr>in && i) {
-		inptr--;
-		if (camel_mime_base64_rank[*inptr] != 0xff) {
-			if (*inptr == '=' && outptr>out)
-				outptr--;
-			i--;
-		}
-	}
-
-	/* if i!= 0 then there is a truncation error! */
-	return outptr-out;
+	return g_base64_decode_step (in, len, out, state, save);
 }
 
+
+/**
+ * camel_base64_encode_simple:
+ * @data: binary stream of data to encode
+ * @len: length of data
+ *
+ * Base64 encodes a block of memory.
+ *
+ * Returns a string containing the base64 encoded data
+ **/
 char *
 camel_base64_encode_simple (const char *data, size_t len)
 {
-	unsigned char *out;
-	int state = 0, outlen;
-	unsigned int save = 0;
-	
-	out = g_malloc (len * 4 / 3 + 5);
-	outlen = camel_base64_encode_close ((unsigned char *)data, len, FALSE,
-				      out, &state, &save);
-	out[outlen] = '\0';
-	return (char *)out;
+	return g_base64_encode (data, len);
 }
 
+
+/**
+ * camel_base64_decode_simple:
+ * @data: data to decode
+ * @len: length of data
+ *
+ * Base64 decodes @data inline (overwrites @data with the decoded version).
+ *
+ * Returns the new length of @data
+ **/
 size_t
 camel_base64_decode_simple (char *data, size_t len)
 {
-	int state = 0;
-	unsigned int save = 0;
+	guchar *out_data;
+	gsize out_len;
 
-	return camel_base64_decode_step ((unsigned char *)data, len,
-				   (unsigned char *)data, &state, &save);
+	out_data = g_base64_decode (data, &out_len);
+	g_assert (out_len <= len); /* sanity check */
+	memcpy (data, out_data, out_len);
+	data[out_len] = '\0';
+	g_free (out_data);
+
+	return out_len;
 }
 
 /**
- * camel_uuencode_close: uuencode a chunk of data
+ * camel_uuencode_close:
  * @in: input stream
  * @len: input stream length
  * @out: output stream
@@ -331,9 +212,10 @@ camel_base64_decode_simple (char *data, size_t len)
  * @state: holds the number of bits that are stored in @save
  * @save: leftover bits that have not yet been encoded
  *
- * Returns the number of bytes encoded. Call this when finished
- * encoding data with camel_uuencode_step to flush off the last little
- * bit.
+ * Uuencodes a chunk of data. Call this when finished encoding data
+ * with #camel_uuencode_step to flush off the last little bit.
+ *
+ * Returns the number of bytes encoded
  **/
 size_t
 camel_uuencode_close (unsigned char *in, size_t len, unsigned char *out, unsigned char *uubuf, int *state, guint32 *save)
@@ -402,7 +284,7 @@ camel_uuencode_close (unsigned char *in, size_t len, unsigned char *out, unsigne
 
 
 /**
- * camel_uuencode_step: uuencode a chunk of data
+ * camel_uuencode_step:
  * @in: input stream
  * @len: input stream length
  * @out: output stream
@@ -410,10 +292,12 @@ camel_uuencode_close (unsigned char *in, size_t len, unsigned char *out, unsigne
  * @state: holds the number of bits that are stored in @save
  * @save: leftover bits that have not yet been encoded
  *
- * Returns the number of bytes encoded. Performs an 'encode step',
- * only encodes blocks of 45 characters to the output at a time, saves
- * left-over state in @uubuf, @state and @save (initialize to 0 on first
+ * Uuencodes a chunk of data. Performs an 'encode step', only encodes
+ * blocks of 45 characters to the output at a time, saves left-over
+ * state in @uubuf, @state and @save (initialize to 0 on first
  * invocation).
+ *
+ * Returns the number of bytes encoded
  **/
 size_t
 camel_uuencode_step (unsigned char *in, size_t len, unsigned char *out, unsigned char *uubuf, int *state, guint32 *save)
@@ -478,16 +362,18 @@ camel_uuencode_step (unsigned char *in, size_t len, unsigned char *out, unsigned
 
 
 /**
- * camel_uudecode_step: uudecode a chunk of data
+ * camel_uudecode_step:
  * @in: input stream
- * @inlen: max length of data to decode ( normally strlen(in) ??)
+ * @inlen: max length of data to decode
  * @out: output stream
  * @state: holds the number of bits that are stored in @save
  * @save: leftover bits that have not yet been decoded
  *
- * Returns the number of bytes decoded. Performs a 'decode step' on
- * a chunk of uuencoded data. Assumes the "begin <mode> <file name>"
- * line has been stripped off.
+ * Uudecodes a chunk of data. Performs a 'decode step' on a chunk of
+ * uuencoded data. Assumes the "begin mode filename" line has
+ * been stripped off.
+ *
+ * Returns the number of bytes decoded
  **/
 size_t
 camel_uudecode_step (unsigned char *in, size_t len, unsigned char *out, int *state, guint32 *save)
@@ -574,9 +460,22 @@ camel_uudecode_step (unsigned char *in, size_t len, unsigned char *out, int *sta
 }
 
 
-/* complete qp encoding */
+/**
+ * camel_quoted_encode_close:
+ * @in: input stream
+ * @len: length of the input
+ * @out: output string
+ * @state: holds the number of bits that are stored in @save
+ * @save: leftover bits that have not yet been encoded
+ *
+ * Quoted-printable encodes a block of text. Call this when finished
+ * encoding data with #camel_quoted_encode_step to flush off
+ * the last little bit.
+ *
+ * Returns the number of bytes encoded
+ **/
 size_t
-camel_quoted_decode_close(unsigned char *in, size_t len, unsigned char *out, int *state, int *save)
+camel_quoted_encode_close(unsigned char *in, size_t len, unsigned char *out, int *state, int *save)
 {
 	register unsigned char *outptr = out;
 	int last;
@@ -603,7 +502,21 @@ camel_quoted_decode_close(unsigned char *in, size_t len, unsigned char *out, int
 	return outptr-out;
 }
 
-/* perform qp encoding, initialise state to -1 and save to 0 on first invocation */
+
+/**
+ * camel_quoted_encode_step:
+ * @in: input stream
+ * @len: length of the input
+ * @out: output string
+ * @state: holds the number of bits that are stored in @save
+ * @save: leftover bits that have not yet been encoded
+ *
+ * Quoted-printable encodes a block of text. Performs an 'encode
+ * step', saves left-over state in state and save (initialise to -1 on
+ * first invocation).
+ *
+ * Returns the number of bytes encoded
+ **/
 size_t
 camel_quoted_encode_step (unsigned char *in, size_t len, unsigned char *out, int *statep, int *save)
 {
@@ -690,6 +603,19 @@ camel_quoted_encode_step (unsigned char *in, size_t len, unsigned char *out, int
   Note: Trailing rubbish (at the end of input), like = or =x or =\r will be lost.
 */ 
 
+/**
+ * camel_quoted_decode_step:
+ * @in: input stream
+ * @len: max length of data to decode
+ * @out: output stream
+ * @savestate: holds the number of bits that are stored in @save
+ * @saved: leftover bits that have not yet been decoded
+ *
+ * Decodes a block of quoted-printable encoded data. Performs a
+ * 'decode step' on a chunk of QP encoded data.
+ *
+ * Returns the number of bytes decoded
+ **/
 size_t
 camel_quoted_decode_step(unsigned char *in, size_t len, unsigned char *out, int *savestate, int *saveme)
 {
@@ -866,8 +792,8 @@ header_decode_lwsp(const char **in)
 
 	d2(printf("is ws: '%s'\n", *in));
 
-	while (camel_mime_is_lwsp(*inptr) || (*inptr =='(' && *inptr != '\0')) {
-		while (camel_mime_is_lwsp(*inptr) && inptr != '\0') {
+	while ((camel_mime_is_lwsp(*inptr) || *inptr =='(') && *inptr != '\0') {
+		while (camel_mime_is_lwsp(*inptr) && *inptr != '\0') {
 			d2(printf("(%c)", *inptr));
 			inptr++;
 		}
@@ -1141,6 +1067,17 @@ header_decode_text (const char *in, size_t inlen, int ctext, const char *default
 	return dword;
 }
 
+
+/**
+ * camel_header_decode_string:
+ * @in: input header value string
+ * @default_charset: default charset to use if improperly encoded
+ *
+ * Decodes rfc2047 encoded-word tokens
+ *
+ * Returns a string containing the UTF-8 version of the decoded header
+ * value
+ **/
 char *
 camel_header_decode_string (const char *in, const char *default_charset)
 {
@@ -1149,6 +1086,18 @@ camel_header_decode_string (const char *in, const char *default_charset)
 	return header_decode_text (in, strlen (in), FALSE, default_charset);
 }
 
+
+/**
+ * camel_header_format_ctext:
+ * @in: input header value string
+ * @default_charset: default charset to use if improperly encoded
+ *
+ * Decodes a header which contains rfc2047 encoded-word tokens that
+ * may or may not be within a comment.
+ *
+ * Returns a string containing the UTF-8 version of the decoded header
+ * value
+ **/
 char *
 camel_header_format_ctext (const char *in, const char *default_charset)
 {
@@ -1181,7 +1130,7 @@ rfc2047_encode_word(GString *outstring, const char *in, size_t len, const char *
 	
 	ascii = g_alloca (bufflen);
 	
-	if (strcasecmp (type, "UTF-8") != 0)
+	if (g_ascii_strcasecmp (type, "UTF-8") != 0)
 		ic = e_iconv_open (type, "UTF-8");
 	
 	while (inlen) {
@@ -1200,10 +1149,10 @@ rfc2047_encode_word(GString *outstring, const char *in, size_t len, const char *
 			/* proclen will be the result of input characters that we can convert, to the nearest
 			   (approximated) valid utf8 char */
 			convlen = 0;
-			proclen = 0;
+			proclen = -1;
 			p = inptr;
 			i = 0;
-			while (p < (in+len) && convlen < (75 - strlen("=?utf-8?q\?\?="))) {
+			while (p < (in+len) && convlen < (75 - strlen("=?utf-8?q?\?="))) {
 				unsigned char c = *p++;
 
 				if (c >= 0xc0)
@@ -1216,8 +1165,10 @@ rfc2047_encode_word(GString *outstring, const char *in, size_t len, const char *
 				else
 					convlen += 3;
 			}
+			if (proclen >= 0 && proclen < i && convlen < (75 - strlen("=?utf-8?q?\?=")))
+				proclen = i;
 			/* well, we probably have broken utf8, just copy it anyway what the heck */
-			if (proclen == 0) {
+			if (proclen == -1) {
 				w(g_warning("Appear to have truncated utf8 sequence"));
 				proclen = inlen;
 			}
@@ -1266,12 +1217,21 @@ rfc2047_encode_word(GString *outstring, const char *in, size_t len, const char *
 
 
 /* TODO: Should this worry about quotes?? */
+/**
+ * camel_header_encode_string:
+ * @in: input string
+ *
+ * Encodes a 'text' header according to the rules of rfc2047.
+ *
+ * Returns the rfc2047 encoded header
+ **/
 char *
 camel_header_encode_string (const unsigned char *in)
 {
 	const unsigned char *inptr = in, *start, *word;
 	gboolean last_was_encoded = FALSE;
 	gboolean last_was_space = FALSE;
+	const char *charset;
 	int encoding;
 	GString *out;
 	char *outstr;
@@ -1335,8 +1295,9 @@ camel_header_encode_string (const unsigned char *in)
 				if (last_was_encoded)
 					g_string_append_c (out, ' ');
 				
-				rfc2047_encode_word (out, start, inptr - start,
-						     camel_charset_best (start, inptr - start), CAMEL_MIME_IS_ESAFE);
+				if (!(charset = camel_charset_best (start, inptr - start)))
+					charset = "UTF-8";
+				rfc2047_encode_word (out, start, inptr - start, charset, CAMEL_MIME_IS_ESAFE);
 				last_was_encoded = TRUE;
 				break;
 			}
@@ -1381,8 +1342,9 @@ camel_header_encode_string (const unsigned char *in)
 			if (last_was_encoded)
 				g_string_append_c (out, ' ');
 			
-			rfc2047_encode_word (out, start, inptr - start,
-					     camel_charset_best (start, inptr - start - 1), CAMEL_MIME_IS_ESAFE);
+			if (!(charset = camel_charset_best (start, inptr - start)))
+				charset = "UTF-8";
+			rfc2047_encode_word (out, start, inptr - start, charset, CAMEL_MIME_IS_ESAFE);
 			break;
 		}
 	}
@@ -1565,11 +1527,20 @@ header_encode_phrase_merge_words (GList **wordsp)
 }
 
 /* encodes a phrase sequence (different quoting/encoding rules to strings) */
+/**
+ * camel_header_encode_phrase:
+ * @in: header to encode
+ *
+ * Encodes a 'phrase' header according to the rules in rfc2047.
+ *
+ * Returns the encoded 'phrase'
+ **/
 char *
 camel_header_encode_phrase (const unsigned char *in)
 {
 	struct _phrase_word *word = NULL, *last_word = NULL;
 	GList *words, *wordl;
+	const char *charset;
 	GString *out;
 	char *outstr;
 	
@@ -1621,11 +1592,13 @@ camel_header_encode_phrase (const unsigned char *in)
 				start = word->start;
 			}
 			
-			if (word->encoding == 1)
+			if (word->encoding == 1) {
 				rfc2047_encode_word (out, start, len, "ISO-8859-1", CAMEL_MIME_IS_PSAFE);
-			else
-				rfc2047_encode_word (out, start, len,
-						     camel_charset_best (start, len), CAMEL_MIME_IS_PSAFE);
+			} else {
+				if (!(charset = camel_charset_best (start, len)))
+					charset = "UTF-8";
+				rfc2047_encode_word (out, start, len, charset, CAMEL_MIME_IS_PSAFE);
+			}
 			break;
 		}
 		
@@ -1666,6 +1639,16 @@ decode_token (const char **in)
 	}
 }
 
+
+/**
+ * camel_header_token_decode:
+ * @in: input string
+ *
+ * Gets the first token in the string according to the rules of
+ * rfc0822.
+ *
+ * Returns a new string containing the first token in @in
+ **/
 char *
 camel_header_token_decode(const char *in)
 {
@@ -1702,12 +1685,15 @@ header_decode_quoted_string(const char **in)
 		}
 		outlen = intmp-inptr-skip;
 		out = outptr = g_malloc(outlen+1);
-		while ( (c = *inptr++) && c!= '"') {
+		while ( (c = *inptr) && c!= '"') {
+			inptr++;
 			if (c=='\\' && *inptr) {
 				c = *inptr++;
 			}
 			*outptr++ = c;
 		}
+		if (c)
+			inptr++;
 		*outptr = '\0';
 	}
 	*in = inptr;
@@ -1763,6 +1749,16 @@ header_decode_value(const char **in)
 }
 
 /* should this return -1 for no int? */
+
+/**
+ * camel_header_decode_int:
+ * @in: pointer to input string
+ *
+ * Extracts an integer token from @in and updates the pointer to point
+ * to after the end of the integer token (sort of like strtol).
+ *
+ * Returns the int value
+ **/
 int
 camel_header_decode_int(const char **in)
 {
@@ -1868,6 +1864,16 @@ rfc2184_decode (const char *in, size_t len)
 	return decoded;
 }
 
+
+/**
+ * camel_header_param:
+ * @params: parameters
+ * @name: name of param to find
+ *
+ * Searches @params for a param named @name and gets the value.
+ *
+ * Returns the value of the @name param
+ **/
 char *
 camel_header_param (struct _camel_header_param *p, const char *name)
 {
@@ -1878,6 +1884,17 @@ camel_header_param (struct _camel_header_param *p, const char *name)
 	return NULL;
 }
 
+
+/**
+ * camel_header_set_param:
+ * @paramsp: poinetr to a list of params
+ * @name: name of param to set
+ * @value: value to set
+ *
+ * Set a parameter in the list.
+ *
+ * Returns the set param
+ **/
 struct _camel_header_param *
 camel_header_set_param (struct _camel_header_param **l, const char *name, const char *value)
 {
@@ -1915,6 +1932,17 @@ camel_header_set_param (struct _camel_header_param **l, const char *name, const
 	return pn;
 }
 
+
+/**
+ * camel_content_type_param:
+ * @content_type: a #CamelContentType
+ * @name: name of param to find
+ *
+ * Searches the params on s #CamelContentType for a param named @name
+ * and gets the value.
+ *
+ * Returns the value of the @name param
+ **/
 const char *
 camel_content_type_param (CamelContentType *t, const char *name)
 {
@@ -1923,41 +1951,58 @@ camel_content_type_param (CamelContentType *t, const char *name)
 	return camel_header_param (t->params, name);
 }
 
+
+/**
+ * camel_content_type_set_param:
+ * @content_type: a #CamelContentType
+ * @name: name of param to set
+ * @value: value of param to set
+ *
+ * Set a parameter on @content_type.
+ **/
 void
 camel_content_type_set_param (CamelContentType *t, const char *name, const char *value)
 {
 	camel_header_set_param (&t->params, name, value);
 }
 
+
 /**
  * camel_content_type_is:
- * @ct: A content type specifier, or #NULL.
+ * @content_type: A content type specifier, or %NULL.
  * @type: A type to check against.
  * @subtype: A subtype to check against, or "*" to match any subtype.
- * 
- * Returns #TRUE if the content type @ct is of type @type/@subtype.
- * The subtype of "*" will match any subtype.  If @ct is #NULL, then
+ *
+ * The subtype of "*" will match any subtype.  If @ct is %NULL, then
  * it will match the type "text/plain".
- * 
- * Return value: #TRUE or #FALSE depending on the matching of the type.
+ *
+ * Returns %TRUE if the content type @ct is of type @type/@subtype or
+ * %FALSE otherwise
  **/
 int
 camel_content_type_is(CamelContentType *ct, const char *type, const char *subtype)
 {
 	/* no type == text/plain or text/"*" */
 	if (ct==NULL || (ct->type == NULL && ct->subtype == NULL)) {
-		return (!strcasecmp(type, "text")
+		return (!g_ascii_strcasecmp(type, "text")
 			&& (!g_ascii_strcasecmp(subtype, "plain")
-			    || !strcasecmp(subtype, "*")));
+			    || !strcmp(subtype, "*")));
 	}
 
 	return (ct->type != NULL
 		&& (!g_ascii_strcasecmp(ct->type, type)
 		    && ((ct->subtype != NULL
 			 && !g_ascii_strcasecmp(ct->subtype, subtype))
-			|| !strcasecmp("*", subtype))));
+			|| !strcmp("*", subtype))));
 }
 
+
+/**
+ * camel_header_param_list_free:
+ * @params: a list of params
+ *
+ * Free the list of params.
+ **/
 void
 camel_header_param_list_free(struct _camel_header_param *p)
 {
@@ -1972,6 +2017,16 @@ camel_header_param_list_free(struct _camel_header_param *p)
 	}
 }
 
+
+/**
+ * camel_content_type_new:
+ * @type: the major type of the new content-type
+ * @subtype: the subtype
+ *
+ * Create a new #CamelContentType.
+ *
+ * Returns the new #CamelContentType
+ **/
 CamelContentType *
 camel_content_type_new(const char *type, const char *subtype)
 {
@@ -1984,6 +2039,13 @@ camel_content_type_new(const char *type, const char *subtype)
 	return t;
 }
 
+
+/**
+ * camel_content_type_ref:
+ * @content_type: a #CamelContentType
+ *
+ * Refs the content type.
+ **/
 void
 camel_content_type_ref(CamelContentType *ct)
 {
@@ -1992,6 +2054,12 @@ camel_content_type_ref(CamelContentType *ct)
 }
 
 
+/**
+ * camel_content_type_unref:
+ * @content_type: a #CamelContentType
+ *
+ * Unrefs, and potentially frees, the content type.
+ **/
 void
 camel_content_type_unref(CamelContentType *ct)
 {
@@ -2001,6 +2069,7 @@ camel_content_type_unref(CamelContentType *ct)
 			g_free(ct->type);
 			g_free(ct->subtype);
 			g_free(ct);
+			ct = NULL;
 		} else {
 			ct->refcount--;
 		}
@@ -2011,20 +2080,19 @@ camel_content_type_unref(CamelContentType *ct)
 static char *
 header_decode_domain(const char **in)
 {
-	const char *inptr = *in, *start;
+	const char *inptr = *in;
 	int go = TRUE;
 	char *ret;
 	GString *domain = g_string_new("");
 
-				/* domain ref | domain literal */
+	/* domain ref | domain literal */
 	header_decode_lwsp(&inptr);
 	while (go) {
 		if (*inptr == '[') { /* domain literal */
 			domain = g_string_append_c(domain, '[');
 			inptr++;
 			header_decode_lwsp(&inptr);
-			start = inptr;
-			while (camel_mime_is_dtext(*inptr)) {
+			while (camel_mime_is_dtext(*inptr) && *inptr) {
 				domain = g_string_append_c(domain, *inptr);
 				inptr++;
 			}
@@ -2202,7 +2270,7 @@ header_decode_mailbox(const char **in, const char *charset)
 				}
 			}
 			pre = header_decode_word (&inptr);
-			header_decode_lwsp(&inptr);
+			/*header_decode_lwsp(&inptr);*/
 		} else {
 			w(g_warning("broken address? %s", *in));
 		}
@@ -2261,10 +2329,10 @@ header_decode_mailbox(const char **in, const char *charset)
 					g_string_truncate(name, 0);
 					g_string_append(name, text);
 				}
-			} else {
+			}/* else {
 				g_string_append(name, text?text:addr->str);
 				g_string_truncate(addr, 0);
-			}
+			}*/
 			g_free(text);
 
 			/* or maybe that we've added up a bunch of broken bits to make an encoded word */
@@ -2433,6 +2501,15 @@ header_msgid_decode_internal(const char **in)
 	return msgid;
 }
 
+
+/**
+ * camel_header_msgid_decode:
+ * @in: input string
+ *
+ * Extract a message-id token from @in.
+ *
+ * Returns the msg-id
+ **/
 char *
 camel_header_msgid_decode(const char *in)
 {
@@ -2442,6 +2519,15 @@ camel_header_msgid_decode(const char *in)
 	return header_msgid_decode_internal(&in);
 }
 
+
+/**
+ * camel_header_contentid_decode:
+ * @in: input string
+ *
+ * Extract a content-id from @in.
+ *
+ * Returns the extracted content-id
+ **/
 char *
 camel_header_contentid_decode (const char *in)
 {
@@ -2943,7 +3029,8 @@ header_encode_param (const unsigned char *in, gboolean *encoded)
 	const char *charset;
 	GString *out;
 	guint32 c;
-
+	char *str;
+	
 	*encoded = FALSE;
 	
 	g_return_val_if_fail (in != NULL, NULL);
@@ -2973,11 +3060,11 @@ header_encode_param (const unsigned char *in, gboolean *encoded)
 	}
 	g_free (outbuf);
 	
-	outbuf = out->str;
+	str = out->str;
 	g_string_free (out, FALSE);
 	*encoded = TRUE;
 	
-	return outbuf;
+	return str;
 }
 
 void
@@ -3110,7 +3197,7 @@ camel_content_type_decode(const char *in)
 			inptr++;
 			subtype = decode_token(&inptr);
 		}
-		if (subtype == NULL && (!strcasecmp(type, "text"))) {
+		if (subtype == NULL && (!g_ascii_strcasecmp(type, "text"))) {
 			w(g_warning("text type with no subtype, resorting to text/plain: %s", in));
 			subtype = g_strdup("plain");
 		}
@@ -3287,12 +3374,12 @@ static struct {
 	{ "Y", 1200 },
 };
 
-static char *tz_months [] = {
+static const char tz_months [][4] = {
 	"Jan", "Feb", "Mar", "Apr", "May", "Jun",
 	"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
 };
 
-static char *tz_days [] = {
+static const char tz_days [][4] = {
 	"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
 };
 
@@ -3576,13 +3663,13 @@ camel_header_raw_append(struct _camel_header_raw **list, const char *name, const
 
 	/* debug */
 #if 0
-	if (!strcasecmp(name, "To")) {
+	if (!g_ascii_strcasecmp(name, "To")) {
 		printf("- Decoding To\n");
 		camel_header_to_decode(value);
-	} else if (!strcasecmp(name, "Content-type")) {
+	} else if (!g_ascii_strcasecmp(name, "Content-type")) {
 		printf("- Decoding content-type\n");
 		camel_content_type_dump(camel_content_type_decode(value));		
-	} else if (!strcasecmp(name, "MIME-Version")) {
+	} else if (!g_ascii_strcasecmp(name, "MIME-Version")) {
 		printf("- Decoding mime version\n");
 		camel_header_mime_decode(value);
 	}
@@ -3805,23 +3892,23 @@ camel_header_raw_check_mailing_list(struct _camel_header_raw **list)
 			match[j].rm_eo = -1;
 		}
 		if (v != NULL && regexec (&mail_list_magic[i].regex, v, 3, match, 0) == 0 && match[1].rm_so != -1) {
-			char *list;
 			int len1, len2;
-
+			char *mlist;
+			
 			len1 = match[1].rm_eo - match[1].rm_so;
 			len2 = match[2].rm_eo - match[2].rm_so;
-
-			list = g_malloc(len1+len2+2);
-			memcpy(list, v + match[1].rm_so, len1);
+			
+			mlist = g_malloc (len1 + len2 + 2);
+			memcpy (mlist, v + match[1].rm_so, len1);
 			if (len2) {
-				list[len1] = '@';
-				memcpy(list+len1+1, v+match[2].rm_so, len2);
-				list[len1+len2+1]=0;
+				mlist[len1] = '@';
+				memcpy (mlist + len1 + 1, v + match[2].rm_so, len2);
+				mlist[len1 + len2 + 1] = '\0';
 			} else {
-				list[len1] = 0;
+				mlist[len1] = '\0';
 			}
-
-			return list;
+			
+			return mlist;
 		}
 	}
 
