@@ -389,7 +389,8 @@ camel_vee_folder_hash_folder(CamelFolder *folder, char buffer[8])
 	g_free(tmp);
 	md5_update(&ctx, folder->full_name, strlen(folder->full_name));
 	md5_final(&ctx, digest);
-	camel_base64_encode_close(digest, 6, FALSE, buffer, &state, &save);
+	g_base64_encode_step(digest, 6, FALSE, buffer, &state, &save);
+	g_base64_encode_close(FALSE, buffer, &state, &save);
 
 	for (i=0;i<8;i++) {
 		if (buffer[i] == '+')
@@ -1443,12 +1444,9 @@ static CamelSessionThreadOps folder_changed_ops = {
 static void
 folder_changed_base(CamelVeeFolder *vf, CamelFolder *sub, CamelFolderChangeInfo *changes)
 {
-	struct _CamelVeeFolderPrivate *p = _PRIVATE(vf);	
 	struct _folder_changed_msg *m;
 	CamelSession *session = ((CamelService *)((CamelFolder *)vf)->parent_store)->session;
-	if (p->destroyed)
- 		return; 
- 	
+	
 	m = camel_session_thread_msg_new(session, &folder_changed_ops, sizeof(*m));
 	m->changes = camel_folder_change_info_new();
 	camel_folder_change_info_cat(m->changes, changes);
@@ -1691,7 +1689,7 @@ camel_vee_folder_init (CamelVeeFolder *obj)
 	CamelFolder *folder = (CamelFolder *)obj;
 
 	p = _PRIVATE(obj) = g_malloc0(sizeof(*p));
-	
+
 	folder->folder_flags |= (CAMEL_FOLDER_HAS_SUMMARY_CAPABILITY |
 				 CAMEL_FOLDER_HAS_SEARCH_CAPABILITY);
 
@@ -1717,8 +1715,6 @@ camel_vee_folder_finalise (CamelObject *obj)
 	struct _CamelVeeFolderPrivate *p = _PRIVATE(vf);
 	CamelVeeFolder *folder_unmatched = vf->parent_vee_store ? vf->parent_vee_store->folder_unmatched : NULL;
 	GList *node;
-	
-	p->destroyed = TRUE;
 
 	/* TODO: there may be other leaks? */
 
