@@ -1,8 +1,8 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@novell.com>
+/*  Camel
+ *  Copyright (C) 1999-2007 Novell, Inc. (www.novell.com)
  *
- *  Copyright 2005 Novell, Inc. (www.novell.com)
+ *  Authors: Jeffrey Stedfast <fejj@novell.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -16,8 +16,7 @@
  *
  *  You should have received a copy of the GNU Lesser General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
+ *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 
@@ -25,16 +24,15 @@
 #include <config.h>
 #endif
 
+#include <ctype.h>
+#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <errno.h>
-#include <ctype.h>
 
-#include <camel/camel-debug.h>
+#include "camel-debug.h"
 
 #include "camel-imap4-specials.h"
-
 #include "camel-imap4-stream.h"
 
 #define d(x) (camel_debug ("imap4:stream") ? (x) : 0)
@@ -469,7 +467,7 @@ camel_imap4_stream_next_token (CamelIMAP4Stream *stream, camel_imap4_token_t *to
 					token->token = CAMEL_IMAP4_TOKEN_LITERAL;
 					token->v.literal = literal;
 					
-					d(fprintf (stderr, "token: {%u}\n", (unsigned int)literal));
+					d(fprintf (stderr, "token: {%lu}\n", literal));
 					
 					stream->mode = CAMEL_IMAP4_STREAM_MODE_LITERAL;
 					stream->literal = literal;
@@ -496,7 +494,7 @@ camel_imap4_stream_next_token (CamelIMAP4Stream *stream, camel_imap4_token_t *to
 				token->token = CAMEL_IMAP4_TOKEN_NUMBER;
 				token->v.number = nz_number;
 				
-				d(fprintf (stderr, "token: %u\n", nz_number));
+				d(fprintf (stderr, "token: %lu\n", nz_number));
 				
 				break;
 			} else if (is_atom (*inptr)) {
@@ -641,6 +639,7 @@ camel_imap4_stream_line (CamelIMAP4Stream *stream, unsigned char **line, size_t
 	inptr = stream->inptr;
 	inend = stream->inend;
 	
+	*inend = '\0';
 	if (inptr == inend || ((inend - inptr) < 2 && *inptr != '\n')) {
 		if (imap4_fill (stream) == -1 && stream->inptr == stream->inend)
 			return -1;
