@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  * USA
  */
 
@@ -23,22 +23,22 @@
 #include <config.h>
 #endif
 
-#include <sys/stat.h>
+#include <dirent.h>
 #include <errno.h>
 #include <string.h>
 #include <unistd.h>
-#include <dirent.h>
+#include <sys/stat.h>
+
+#include <glib/gi18n-lib.h>
 
-#include "camel-mh-store.h"
-#include "camel-mh-folder.h"
 #include "camel-exception.h"
-#include "camel-url.h"
 #include "camel-private.h"
-#include "camel-i18n.h"
-
-#include <camel/camel-stream-fs.h>
-#include <camel/camel-stream-buffer.h>
+#include "camel-stream-buffer.h"
+#include "camel-stream-fs.h"
+#include "camel-url.h"
 
+#include "camel-mh-folder.h"
+#include "camel-mh-store.h"
 #include "camel-mh-summary.h"
 
 static CamelLocalStoreClass *parent_class = NULL;
@@ -165,7 +165,7 @@ folders_update(const char *root, int mode, const char *folder, const char *new)
 				if (camel_stream_printf(out, "%s\n", folder) == -1)
 					goto fail;
 				mode = UPDATE_NONE;
-			} else if (tmp == 0) {
+			} else if (tmp == NULL) {
 				/* already there */
 				mode = UPDATE_NONE;
 			}
