@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /* ExchangeAccount: Handles a single configured Connector account. This
@@ -32,9 +32,7 @@
 #include "exchange-folder-size.h"
 #include "e-folder-exchange.h"
 #include "e2k-autoconfig.h"
-#include "e2k-encoding-utils.h"
 #include "e2k-kerberos.h"
-#include "e2k-marshal.h"
 #include "e2k-propnames.h"
 #include "e2k-uri.h"
 #include "e2k-utils.h"
@@ -54,7 +52,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#define d(x)
+#define d(x) 
 #define ADS_UF_DONT_EXPIRE_PASSWORD 0x10000
 #define ONE_HUNDRED_NANOSECOND 0.000000100
 #define SECONDS_IN_DAY 86400
@@ -121,7 +119,7 @@ class_init (GObjectClass *object_class)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (ExchangeAccountClass, connected),
 			      NULL, NULL,
-			      e2k_marshal_NONE__OBJECT,
+			      g_cclosure_marshal_VOID__OBJECT,
 			      G_TYPE_NONE, 1,
 			      E2K_TYPE_CONTEXT);
 	signals[NEW_FOLDER] =
@@ -130,7 +128,7 @@ class_init (GObjectClass *object_class)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (ExchangeAccountClass, new_folder),
 			      NULL, NULL,
-			      e2k_marshal_NONE__POINTER,
+			      g_cclosure_marshal_VOID__POINTER,
 			      G_TYPE_NONE, 1,
 			      G_TYPE_POINTER);
 	signals[REMOVED_FOLDER] =
@@ -139,7 +137,7 @@ class_init (GObjectClass *object_class)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (ExchangeAccountClass, removed_folder),
 			      NULL, NULL,
-			      e2k_marshal_NONE__POINTER,
+			      g_cclosure_marshal_VOID__POINTER,
 			      G_TYPE_NONE, 1,
 			      G_TYPE_POINTER);
 }
@@ -527,6 +525,8 @@ exchange_account_remove_folder (ExchangeAccount *account, const char *path)
 	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), 
 				EXCHANGE_ACCOUNT_FOLDER_GENERIC_ERROR);
 
+	d(g_print ("exchange_account_remove_folder: path=[%s]\n", path));
+
 	if (!get_folder (account, path, &folder, &hier))
 		return EXCHANGE_ACCOUNT_FOLDER_DOES_NOT_EXIST;
 
@@ -783,6 +783,8 @@ exchange_account_open_folder (ExchangeAccount *account, const char *path)
 	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), 
 				EXCHANGE_ACCOUNT_FOLDER_GENERIC_ERROR);
 
+	d(g_print ("exchange_account_remove_folder: path=[%s]\n", path));
+
 	if (!get_folder (account, path, &folder, &hier))
 		return EXCHANGE_ACCOUNT_FOLDER_DOES_NOT_EXIST;
 
@@ -888,6 +890,8 @@ static const char *mailbox_info_props[] = {
 };
 static const int n_mailbox_info_props = G_N_ELEMENTS (mailbox_info_props);
 
+
+
 static gboolean
 account_moved (ExchangeAccount *account, E2kAutoconfig *ac)
 {
@@ -1244,8 +1248,7 @@ setup_account_hierarchies (ExchangeAccount *account)
 					      account->priv->identity_name,
 					      account->priv->identity_email,
 					      account->priv->source_uri,
-					      account->priv->offline_sync);
-	d(g_print ("exchange-account.c:setup_account_hierarchies:offline_sync=%d\n", account->priv->offline_sync));
+					      TRUE);
 
 	setup_hierarchy (account, hier);
 	g_free (phys_uri_prefix);
@@ -1277,7 +1280,7 @@ setup_account_hierarchies (ExchangeAccount *account)
 					      account->priv->identity_name,
 					      account->priv->identity_email,
 					      account->priv->source_uri,
-					      account->priv->offline_sync);
+					      FALSE);
 	setup_hierarchy (account, hier);
 	g_free (phys_uri_prefix);
 
@@ -1362,7 +1365,7 @@ exchange_account_connect (ExchangeAccount *account, const char *pword,
 	E2kResult *results;
 	int nresults = 0, mode;
 	GByteArray *entryid;
-	const char *timezone;
+	const char *tz;
 	E2kGlobalCatalogStatus gcstatus;
 	E2kGlobalCatalogEntry *entry;
 	E2kOperation gcop;
@@ -1373,9 +1376,16 @@ exchange_account_connect (ExchangeAccount *account, const char *pword,
 
 	*info_result = EXCHANGE_ACCOUNT_CONNECT_SUCCESS;
 	exchange_account_is_offline (account, &mode);
-
+	
 	g_mutex_lock (account->priv->connect_lock);
 
+	if (mode == UNSUPPORTED_MODE) {
+		*info_result = EXCHANGE_ACCOUNT_CONNECT_ERROR;
+		account->priv->connecting = FALSE;
+		g_mutex_unlock (account->priv->connect_lock);
+		return NULL;
+	}
+	
 	if (account->priv->connecting || mode == OFFLINE_MODE) {
 		g_mutex_unlock (account->priv->connect_lock);
 		if (mode == OFFLINE_MODE) {
@@ -1536,9 +1546,9 @@ exchange_account_connect (ExchangeAccount *account, const char *pword,
 		if (entryid)
 			account->legacy_exchange_dn = g_strdup (e2k_entryid_to_dn (entryid));
 
-		timezone = e2k_properties_get_prop (results[0].props, E2K_PR_EXCHANGE_TIMEZONE);
-		if (timezone)
-			account->default_timezone = g_strdup (timezone);
+		tz = e2k_properties_get_prop (results[0].props, E2K_PR_EXCHANGE_TIMEZONE);
+		if (tz)
+			account->default_timezone = g_strdup (tz);
 	}
 
 	if (!setup_account_hierarchies (account)) {
@@ -1643,6 +1653,21 @@ exchange_account_get_global_catalog (ExchangeAccount *account)
 	return account->priv->gc;
 }
 
+/**
+ * exchange_account_fetch:
+ * @acct: an #ExchangeAccount
+ *
+ * Return value: @account's #EAccount, if it is connected and
+ * online, or %NULL if not.
+ **/	
+EAccount *
+exchange_account_fetch (ExchangeAccount *acct)
+{
+	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (acct), NULL);
+
+	return acct->priv->account;
+}
+
 /**
  * exchange_account_get_standard_uri:
  * @account: an #ExchangeAccount
@@ -1756,6 +1781,71 @@ exchange_account_get_foreign_uri (ExchangeAccount *account,
 	return foreign_uri;
 }
 
+/* Scans the subscribed users folders. */
+/*FIXME This function is not really required if the syncronization
+  problem between exchange and evolution is fixed. Exchange does not get to know 
+ if an user's folder is subscribed from evolution */
+void
+exchange_account_scan_foreign_hierarchy (ExchangeAccount *account, const char *user_email)
+{
+	char *dir;
+	ExchangeHierarchy *hier;
+	int mode;
+	
+	hier = g_hash_table_lookup (account->priv->foreign_hierarchies, user_email);
+	if (hier) {
+		exchange_hierarchy_rescan (hier);
+		return;
+	}
+
+	dir = g_strdup_printf ("%s/%s", account->storage_dir, user_email);
+	if (g_file_test (dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) {
+		hier = exchange_hierarchy_foreign_new_from_dir (account, dir);
+		g_free (dir);
+		if (hier) {
+			exchange_account_is_offline (account, &mode);
+			setup_hierarchy_foreign (account, hier);
+			exchange_hierarchy_scan_subtree (hier, hier->toplevel, mode);
+		}
+	}
+}
+
+/**
+ * exchange_account_get_hierarchy_by_email:
+ * @account: an #ExchangeAccount
+ * @email: email id of the foreign user
+ *
+ * If the hierarchy is present just return it back. Else try to get it
+ * from the filesystem and return it. 
+ *
+ * Return value: Returns the ExchangeHierarchy of the foreign user's folder.
+ **/
+
+ExchangeHierarchy *
+exchange_account_get_hierarchy_by_email (ExchangeAccount *account, const char *email)
+{
+	char *dir;
+	ExchangeHierarchy *hier = NULL;
+	int mode;
+
+	g_return_val_if_fail (email != NULL, NULL);
+
+	hier = g_hash_table_lookup (account->priv->foreign_hierarchies, email);
+	if (!hier) {
+		dir = g_strdup_printf ("%s/%s", account->storage_dir, email);
+		if (g_file_test (dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) {
+			hier = exchange_hierarchy_foreign_new_from_dir (account, dir);
+			g_free (dir);
+			if (hier) {
+				exchange_account_is_offline (account, &mode);
+				setup_hierarchy_foreign (account, hier);
+			}
+		}
+	}
+
+	return hier;
+ }
+
 /**
  * exchange_account_get_folder:
  * @account: an #ExchangeAccount
@@ -1785,11 +1875,20 @@ folder_comparator (const void *a, const void *b)
 		       e_folder_exchange_get_path (*fb));
 }
 
+struct _folders_tree {
+	char *path;
+	GPtrArray *folders;
+};
+
+
 static void
 add_folder (gpointer key, gpointer value, gpointer folders)
 {
 	EFolder *folder = value;
 
+	d(g_print ("%s(%d):%s: key=[%s]\t folder-path=[%s]\n", __FILE__, __LINE__, G_GNUC_PRETTY_FUNCTION, 
+		   key, e_folder_exchange_get_path (folder)));
+
 	/* Each folder appears under three different keys, but
 	 * we only want to add it to the results array once. So
 	 * we only add when we see the "path" key.
@@ -1798,6 +1897,19 @@ add_folder (gpointer key, gpointer value, gpointer folders)
 		g_ptr_array_add (folders, folder);
 }
 
+static void
+add_folder_tree (gpointer key, gpointer value, gpointer folders)
+{
+	EFolder *folder = value;
+	struct _folders_tree *fld_tree = (struct _folders_tree *) folders;
+
+	if (!fld_tree || !fld_tree->path)
+		return;
+
+	if (g_str_has_prefix (key, fld_tree->path))
+		add_folder (key, folder, fld_tree->folders);
+}
+
 /**
  * exchange_account_get_folders:
  * @account: an #ExchangeAccount
@@ -1818,9 +1930,10 @@ exchange_account_get_folders (ExchangeAccount *account)
 	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), NULL);
 
 	folders = g_ptr_array_new ();
-	if (account->priv->fresh_folders)
+	/*	if (account->priv->fresh_folders)
 		g_hash_table_foreach (account->priv->fresh_folders, add_folder, folders);
 	else
+	*/
 		g_hash_table_foreach (account->priv->folders, add_folder, folders);
 
 	qsort (folders->pdata, folders->len,
@@ -1829,6 +1942,53 @@ exchange_account_get_folders (ExchangeAccount *account)
 	return folders;
 }	
 
+/**
+ * exchange_account_get_folder_tree:
+ * @account: an #ExchangeAccount
+ *
+ * Return an array of folders (sorted such that parents will occur
+ * before children). If the caller wants to keep up to date with the
+ * list of folders, he should also listen to %new_folder and
+ * %removed_folder signals.
+ *
+ * Return value: an array of folders. The array should be freed with
+ * g_ptr_array_free().
+ **/
+GPtrArray *
+exchange_account_get_folder_tree (ExchangeAccount *account, char* path)
+{
+	GPtrArray *folders = NULL;
+	EFolder *folder = NULL;
+	ExchangeHierarchy *hier = NULL;
+
+	struct _folders_tree *fld_tree = NULL;
+
+	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), NULL);
+
+	if (!get_folder (account, path, &folder, &hier))
+		return folders;
+
+	exchange_hierarchy_scan_subtree (hier, folder, account->priv->account_online);
+
+	folders = g_ptr_array_new ();
+	fld_tree = g_new0 (struct _folders_tree, 1);
+	fld_tree->path = path;
+	fld_tree->folders = folders;
+
+	/*	if (account->priv->fresh_folders)
+		g_hash_table_foreach (account->priv->fresh_folders, add_folder, folders);
+	else
+	*/
+	g_hash_table_foreach (account->priv->folders, add_folder_tree, fld_tree);
+
+	qsort (folders->pdata, folders->len,
+	       sizeof (EFolder *), folder_comparator);
+
+ 	g_free (fld_tree);
+
+	return folders;
+}	
+
 /**
  * exchange_account_get_quota_limit:
  * @account: an #ExchangeAccount
@@ -1848,7 +2008,7 @@ exchange_account_get_quota_limit (ExchangeAccount *account)
 int
 exchange_account_check_password_expiry (ExchangeAccount *account)
 {
-	E2kGlobalCatalogEntry *entry;
+	E2kGlobalCatalogEntry *entry=NULL; /* This is never set before it's used! */
 	int max_pwd_age_days = -1;
 
 	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), 0);
@@ -2140,3 +2300,29 @@ exchange_account_new (EAccountList *account_list, EAccount *adata)
 
 	return account;
 }
+
+/**
+ * exchange_account_get_hierarchy_by_type:
+ * @account: an #ExchangeAccount
+ * @type: Hierarchy type
+ *
+ * Returns the non-foreign hierarchy pointer for the requested type 
+ *
+ * Return value: Returns the hierarchy pointer for the requested type
+ **/
+
+ExchangeHierarchy*
+exchange_account_get_hierarchy_by_type (ExchangeAccount* acct, 
+					ExchangeHierarchyType type)
+{
+	int i;
+	
+	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (acct), NULL);
+	g_return_val_if_fail (type != EXCHANGE_HIERARCHY_FOREIGN, NULL);
+
+	for (i = 0; i < acct->priv->hierarchies->len; i++) {
+		if (EXCHANGE_HIERARCHY (acct->priv->hierarchies->pdata[i])->type == type)
+			return EXCHANGE_HIERARCHY (acct->priv->hierarchies->pdata[i]);
+	}
+	return NULL;
+}
